/*
 * SimpleDoublePrecisionJuliaGenerator
 * 
 * Version 0.2alpha
 * 
 * 2010-06-11
 * 
 * CopyLeft
 */
package infpp.fractal;

/**
 * The SimpleDoublePrecisionJuliaGenerator provides an implementation for
 * calculating the number of required iterations to determine the divergence of
 * the sequence of points obtained by the recursion
 * z<sub>n + 1</sub> = (z<sub>n</sub>)<sup>2</sup> + c
 * where c is the point passed to the setDefiningPoint method and is constant
 * during the calculation of the fractal and z<sub>1</sub> is a point
 * determined by screen coordinates.  This implementation operates with
 * standard double precision arithmetic.
 * 
 * @author both
 */
public class SimpleDoublePrecisionJuliaGenerator
        extends DoublePrecisionFractalGenerator
        implements JuliaFractalGenerator {
    private int     maxIterations;
    private Complex definingPoint = new Complex(0, 0);

    /**
     * Constructs a new SimpleDoublePrecisionJuliaGenerator initialized with
     * (0, 0) as the defining point.
     * 
     * @param maxIterations the maximum number of iterations to use
     */
    public SimpleDoublePrecisionJuliaGenerator(int maxIterations) {
        this.maxIterations = maxIterations;
    }

    /**
     * @see JuliaFractalGenerator#getDefiningPoint()
     */
    @Override
    public Complex getDefiningPoint() {
        return this.definingPoint;
    }

    /**
     * @see JuliaFractalGenerator#setDefiningPoint(Complex)
     */
    @Override
    public void setDefiningPoint(Complex definingPoint) {
        this.definingPoint = definingPoint;
    }

    /**
     * Calculates the number of required iterations to determine the divergence
     * of the sequence of points obtained by the recursion
     * z<sub>n + 1</sub> = (z<sub>n</sub>)<sup>2</sup> + c
     * where c is the point passed to the setDefiningPoint method and is
     * constant during the calculation of the fractal and z<sub>1</sub> is a
     * point determined by screen coordinates.
     * 
     * @param  x the x coordinate of the specified pixel
     * @param  y the y coordinate of the specified pixel
     * @return the number of required iterations
     * @see    FractalGenerator#getNumberOfIterationsAtPixel(int, int)
     */
    @Override
    public int getNumberOfIterationsAtPixel(int x, int y) {
        Complex z = calculateComplexFromScreenCoordinates(x, y);
        int i;
        for (i = 0; i < this.maxIterations && z.getAbsSquare() <= 4; ++i) {
            z.square();
            z.addTo(this.getDefiningPoint());
        }
        return i;
    }
}
