/*
 * SwingFractalGUI
 * 
 * Version 0.2alpha
 * 
 * 2010-06-11
 * 
 * CopyLeft
 */
package infpp.fractal;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

/**
 * The SwingFractalGUI class provides a graphical user interface to a set of a
 * Julia and a Mandelbrot FractalGenerator.
 * 
 * @author both
 */
public final class SwingFractalGUI extends JFrame {
    private enum PanelType {
	NONE, MANDELBROT, JULIA
    };

    /** */
    private static final long serialVersionUID = 1L;

    private FractalGenerator      mandelbrotGenerator;
    private JuliaFractalGenerator juliaGenerator;
    private Colorizer             mandelbrotColorizer;
    private Colorizer             juliaColorizer;
    final private JPanel          mandelbrotPanel;
    private int                   mandelbrotPanelWidth;
    private int                   mandelbrotPanelHeight;
    final private JFrame          juliaFrame;
    final private JPanel          juliaPanel;
    private int                   juliaPanelWidth;
    private int                   juliaPanelHeight;
    private PanelType             dragPanelType = PanelType.NONE;
    private Point                 initialDragPosition;
    private Point                 currentDragPosition;


    /**
     * Constructs a SwingFractalGUI using the supplied FractalGenerators and
     * Colorizers and default values for width and height.
     * 
     * @param mandelbrotGenerator the supplied Mandelbrot FractalGenerator
     * @param juliaGenerator      the supplied JuliaFractalGenerator
     * @param mandelbrotColorizer the supplied Colorizer for displaying the
     *                            Mandelbrot fractal
     * @param juliaColorizer      the supplied Colorizer for displaying the
     *                            Julia fractal
     */
    public SwingFractalGUI(FractalGenerator mandelbrotGenerator,
            JuliaFractalGenerator juliaGenerator,
            Colorizer mandelbrotColorizer, Colorizer juliaColorizer) {
        this(mandelbrotGenerator, juliaGenerator, mandelbrotColorizer,
                juliaColorizer, 630, 670);
    }


    /**
     * Constructs a SwingFractalGUI using the supplied generators and
     * Colorizers as well as the specified values for width, height.
     * 
     * @param mandelbrotGenerator the supplied Mandelbrot FractalGenerator
     * @param juliaGenerator      the supplied JuliaFractalGenerator
     * @param mandelbrotColorizer the supplied Colorizer for displaying the
     *                            Mandelbrot fractal
     * @param juliaColorizer      the supplied Colorizer for displaying the
     *                            Julia fractal
     * @param width               the width of the window
     * @param height              the height of the window
     */
    public SwingFractalGUI(FractalGenerator mandelbrotGenerator,
            JuliaFractalGenerator juliaGenerator,
            Colorizer mandelbrotColorizer, Colorizer juliaColorizer, int width,
            int height) {
        this.mandelbrotGenerator = mandelbrotGenerator;
        this.juliaGenerator = juliaGenerator;
        this.mandelbrotColorizer = mandelbrotColorizer;
        this.juliaColorizer = juliaColorizer;


        // ################ Mandelbrot frame ################
        super.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        super.setSize(width, height);
        super.setLocation(0, 0);
        this.updateMandelbrotTitle();

        mandelbrotPanel = new JPanel(false) {
            /** the generated serial version UID */
            private static final long serialVersionUID = -8962932890088924931L;

            @Override
            public void paintComponent(Graphics g) {
                drawMandelbrotFractal(g);
            }
        };
        super.add(this.mandelbrotPanel);

        super.addComponentListener(new ComponentAdapter() {
            @Override
            public void componentResized(ComponentEvent e) {
                mandelbrotPanelWidth = Math.max(1, mandelbrotPanel.getWidth());
                mandelbrotPanelHeight
                        = Math.max(1, mandelbrotPanel.getHeight());
                getMandelbrotGenerator().setScreenDimension(
                        mandelbrotPanelWidth, mandelbrotPanelHeight);
                
            }
        });


        // ################ Julia frame ################
        this.juliaFrame = new JFrame();
        this.juliaFrame.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
        this.juliaFrame.setSize(width, height);
        this.juliaFrame.setLocation(width + 10, 0);

        juliaPanel = new JPanel(false) {
            /** the generated serial version UID */
            private static final long serialVersionUID = 1985592309713529010L;

            @Override
            public void paintComponent(Graphics g) {
                drawJuliaFractal(g);
            }
        };
        this.juliaFrame.add(juliaPanel);

        this.juliaFrame.addComponentListener(new ComponentAdapter() {
            @Override
            public void componentResized(ComponentEvent e) {
                juliaPanelWidth = Math.max(1, juliaPanel.getWidth());
                juliaPanelHeight = Math.max(1, juliaPanel.getHeight());
                getJuliaGenerator().setScreenDimension(juliaPanelWidth,
                        juliaPanelHeight);
            }
        });


        // ################ interaction ################
        MandelbrotMouseListener mandelbrotMouseListener
                = new MandelbrotMouseListener();
        this.mandelbrotPanel.addMouseListener(mandelbrotMouseListener);
        this.mandelbrotPanel.addMouseMotionListener(mandelbrotMouseListener);
        this.mandelbrotPanel.addMouseWheelListener(mandelbrotMouseListener);
        this.defineMandelbrotKeys();

        JuliaMouseListener juliaMouseListener = new JuliaMouseListener();
        this.juliaPanel.addMouseListener(juliaMouseListener);
        this.juliaPanel.addMouseMotionListener(juliaMouseListener);
        this.juliaPanel.addMouseWheelListener(juliaMouseListener);
        this.defineJuliaKeys();


        super.setVisible(true);


        // ################ the continuously repainting thread ################
        new Thread() {
            @Override
            public void run() {
                while (mandelbrotPanel.isVisible()) {
                    mandelbrotPanel.repaint();
                    juliaPanel.repaint();
                    try {
                        Thread.sleep(400);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
            }
        }.start();
    }


    /**
     * The FractalMouseListener serves as a base class for mouse listeners
     * employed by the SwingFractalGUI.
     */
    class FractalMouseListener extends MouseInputAdapter
            implements MouseWheelListener {}


    /**
     * The MandelbrotMouseListener is the mouse listener that handles mouse
     * events on the mandelbrotPanel.
     */
    class MandelbrotMouseListener extends FractalMouseListener {
        @Override
        public void mousePressed(MouseEvent e) {
            switch (e.getButton()) {
            case MouseEvent.BUTTON1:
                // start dragging rectangle for zoom to selection
                initialDragPosition = e.getPoint();
                currentDragPosition = e.getPoint();
                dragPanelType = PanelType.MANDELBROT;
        	break;

            case MouseEvent.BUTTON2:
                // set defining point for Julia fractal generator
                Complex definingPoint = mandelbrotGenerator
                        .calculateComplexFromScreenCoordinates(e.getX(),
                                e.getY());
                setJuliaDefiningPoint(definingPoint);
                juliaFrame.setVisible(true);
                juliaPanel.repaint();
        	break;

            case MouseEvent.BUTTON3:
                // jump to selected position
        	Complex newOrigin = mandelbrotGenerator
        	        .calculateComplexFromScreenCoordinates(e.getPoint().x,
        	        	e.getPoint().y);
        	setMandelbrotComplexOrigin(newOrigin);
            }
        }

        @Override
	public void mouseDragged(MouseEvent e) {
            // drag rectangle for zoom to selection
	    currentDragPosition = e.getPoint();
	    
	}

        @Override
        public void mouseReleased(MouseEvent e) {
            switch (e.getButton()) {
            case MouseEvent.BUTTON1:
        	// finish dragging rectangle and zoom to selection
                dragPanelType = PanelType.NONE;
                currentDragPosition = e.getPoint();

                if (initialDragPosition.distance(currentDragPosition) < 4)
                    break;

                Complex complexInitialDragPosition = mandelbrotGenerator
                        .calculateComplexFromScreenCoordinates(
                        	initialDragPosition.x, initialDragPosition.y);

                Complex complexFinalDragPosition = mandelbrotGenerator
                        .calculateComplexFromScreenCoordinates(
    	                        currentDragPosition.x, currentDragPosition.y);

                Complex newOrigin = new Complex(complexInitialDragPosition);
                newOrigin.addTo(complexFinalDragPosition);
                newOrigin.multiplyBy(new Complex(.5, 0));

                double newDimension = Math.max(
                	Math.abs(complexInitialDragPosition.re
                                - complexFinalDragPosition.re),
                	Math.abs(complexInitialDragPosition.im
                	        - complexFinalDragPosition.im));

                setMandelbrotComplexOrigin(newOrigin);
                setMandelbrotRealDimension(newDimension);
            }
        }

        @Override
        public void mouseWheelMoved(MouseWheelEvent e) {
            if (e.getWheelRotation() > 0) {
        	// zoom in
		setMandelbrotRealDimension(
			mandelbrotGenerator.getRealDimension() * .875);
            } else if (e.getWheelRotation() < 0) {
        	// zoom out
		setMandelbrotRealDimension(
			mandelbrotGenerator.getRealDimension() * 1.142857);
            }
        }
    }


    /**
     * The JuliaMouseListener is the mouse listener that handles mouse events
     * on the juliaPanel.
     */
    class JuliaMouseListener extends FractalMouseListener {
        @Override
        public void mousePressed(MouseEvent e) {
            switch (e.getButton()) {
            case MouseEvent.BUTTON1:
                // start dragging rectangle for zoom to selection
                initialDragPosition = e.getPoint();
                currentDragPosition = e.getPoint();
                dragPanelType = PanelType.JULIA;
        	break;

//            case MouseEvent.BUTTON2:
//                // set defining point for Julia fractal generator
//                Complex definingPoint = mandelbrotGenerator
//                        .calculateComplexFromScreenCoordinates(e.getX(),
//                                e.getY());
//                setJuliaDefiningPoint(definingPoint);
//                juliaFrame.setVisible(true);
//                juliaPanel.repaint();
//        	break;

            case MouseEvent.BUTTON3:
                // jump to selected position
        	Complex newOrigin = juliaGenerator
        	        .calculateComplexFromScreenCoordinates(e.getPoint().x,
        	        	e.getPoint().y);
        	setJuliaComplexOrigin(newOrigin);
            }
        }

        @Override
	public void mouseDragged(MouseEvent e) {
            // drag rectangle for zoom to selection
	    currentDragPosition = e.getPoint();
	    
	}

        @Override
        public void mouseReleased(MouseEvent e) {
            switch (e.getButton()) {
            case MouseEvent.BUTTON1:
        	// finish dragging rectangle and zoom to selection
                dragPanelType = PanelType.NONE;
                currentDragPosition = e.getPoint();

                if (initialDragPosition.distance(currentDragPosition) < 4)
                    break;

                Complex complexInitialDragPosition = juliaGenerator
                        .calculateComplexFromScreenCoordinates(
                        	initialDragPosition.x, initialDragPosition.y);

                Complex complexFinalDragPosition = juliaGenerator
                        .calculateComplexFromScreenCoordinates(
    	                        currentDragPosition.x, currentDragPosition.y);

                Complex newOrigin = new Complex(complexInitialDragPosition);
                newOrigin.addTo(complexFinalDragPosition);
                newOrigin.multiplyBy(new Complex(.5, 0));

                double newDimension = Math.max(
                	Math.abs(complexInitialDragPosition.re
                                - complexFinalDragPosition.re),
                	Math.abs(complexInitialDragPosition.im
                	        - complexFinalDragPosition.im));

                setJuliaComplexOrigin(newOrigin);
                setJuliaRealDimension(newDimension);
            }
        }

        @Override
        public void mouseWheelMoved(MouseWheelEvent e) {
            if (e.getWheelRotation() > 0) {
        	// zoom in
		setJuliaRealDimension(
			juliaGenerator.getRealDimension() * .875);
            } else if (e.getWheelRotation() < 0) {
        	// zoom out
		setJuliaRealDimension(
			juliaGenerator.getRealDimension() * 1.142857);
            }
        }
    }


    /**
     * Defines the key mappings for the frame displaying the Mandelbrot
     * fractal.
     */
    private void defineMandelbrotKeys() {
	InputMap im = super.getRootPane().getInputMap();
	ActionMap am = super.getRootPane().getActionMap();

	// reset position
	am.put("Action_for_0_Key", new AbstractAction() {
	    /** the generated serial version UID */
	    private static final long serialVersionUID = -3362005230810527677L;

	    @Override
	    public void actionPerformed(ActionEvent e) {
		setMandelbrotComplexOrigin(new Complex(0, 0));
		setMandelbrotRealDimension(4);
	    }
	});
	im.put(KeyStroke.getKeyStroke('0'), "Action_for_0_Key");

	// move left
	am.put("Action_for_Left_Arrow_Key", new AbstractAction() {
	    /** the generated serial version UID */
	    private static final long serialVersionUID = -8260155757215183876L;

	    @Override
	    public void actionPerformed(ActionEvent e) {
		Complex newOrigin
		        = new Complex(mandelbrotGenerator.getComplexOrigin());
		newOrigin.addTo(new Complex(
			-mandelbrotGenerator.getRealDimension() * .0625, 0));
		setMandelbrotComplexOrigin(newOrigin);
	    }
	});
	im.put(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 0),
		"Action_for_Left_Arrow_Key");

	// move right
	am.put("Action_for_Right_Arrow_Key", new AbstractAction() {
	    /** the generated serial version UID */
	    private static final long serialVersionUID = 6671663504426998800L;

	    @Override
	    public void actionPerformed(ActionEvent e) {
		Complex newOrigin
		        = new Complex(mandelbrotGenerator.getComplexOrigin());
		newOrigin.addTo(new Complex(
			mandelbrotGenerator.getRealDimension() * .0625, 0));
		setMandelbrotComplexOrigin(newOrigin);
	    }
	});
	im.put(KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 0),
		"Action_for_Right_Arrow_Key");

	// move up
	am.put("Action_for_Up_Arrow_Key", new AbstractAction() {
	    /** the generated serial version UID */
	    private static final long serialVersionUID = -6929086432023580556L;

	    @Override
	    public void actionPerformed(ActionEvent e) {
		Complex newOrigin
		        = new Complex(mandelbrotGenerator.getComplexOrigin());
		newOrigin.addTo(new Complex(
			0, mandelbrotGenerator.getRealDimension() * .0625));
		setMandelbrotComplexOrigin(newOrigin);
	    }
	});
	im.put(KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0),
		"Action_for_Up_Arrow_Key");

	// move down
	am.put("Action_for_Down_Arrow_Key", new AbstractAction() {
	    /** the generated serial version UID */
	    private static final long serialVersionUID = 2245326940677181354L;

	    @Override
	    public void actionPerformed(ActionEvent e) {
		Complex newOrigin
		        = new Complex(mandelbrotGenerator.getComplexOrigin());
		newOrigin.addTo(new Complex(
			0, -mandelbrotGenerator.getRealDimension() * .0625));
		setMandelbrotComplexOrigin(newOrigin);
	    }
	});
	im.put(KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 0),
		"Action_for_Down_Arrow_Key");

	// zoom in
	am.put("Action_for_+_Key", new AbstractAction() {
	    /** the generated serial version UID */
	    private static final long serialVersionUID = 3434995494597910699L;

	    @Override
	    public void actionPerformed(ActionEvent e) {
		setMandelbrotRealDimension(
			mandelbrotGenerator.getRealDimension() * .875);
	    }
	});
	im.put(KeyStroke.getKeyStroke('+'), "Action_for_+_Key");

	// zoom out
	am.put("Action_for_-_Key", new AbstractAction() {
	    /** the generated serial version UID */
	    private static final long serialVersionUID = 8495499764904290734L;

	    @Override
	    public void actionPerformed(ActionEvent e) {
		setMandelbrotRealDimension(
			mandelbrotGenerator.getRealDimension() * 1.142857);
	    }
	});
	im.put(KeyStroke.getKeyStroke('-'), "Action_for_-_Key");
	
	// show dialogue
	am.put("Action_for_Ctrl_Key", new AbstractAction() {
	    /** the generated serial version UID */
	    private static final long serialVersionUID = 8495499764904290734L;

	    @Override
	    public void actionPerformed(ActionEvent e) {
//		System.out.println("ctrl");
//		showDialogue(); // yet to be implemented
	    }
	});
	im.put(KeyStroke.getKeyStroke(KeyEvent.VK_CONTROL,
		InputEvent.CTRL_DOWN_MASK), "Action_for_Ctrl_Key");
    }


    /**
     * Defines the key mappings for the frame displaying the Julia fractal.
     */
    private void defineJuliaKeys() {
	InputMap im = this.juliaPanel.getRootPane().getInputMap();
	ActionMap am = this.juliaPanel.getRootPane().getActionMap();

	// reset position
	am.put("Action_for_0_Key", new AbstractAction() {
	    /** the generated serial version UID */
	    private static final long serialVersionUID = -3362005230810527677L;

	    @Override
	    public void actionPerformed(ActionEvent e) {
		setJuliaComplexOrigin(new Complex(0, 0));
		setJuliaRealDimension(4);
	    }
	});
	im.put(KeyStroke.getKeyStroke('0'), "Action_for_0_Key");

	// move left
	am.put("Action_for_Left_Arrow_Key", new AbstractAction() {
	    /** the generated serial version UID */
	    private static final long serialVersionUID = -8260155757215183876L;

	    @Override
	    public void actionPerformed(ActionEvent e) {
		Complex newOrigin
		        = new Complex(juliaGenerator.getComplexOrigin());
		newOrigin.addTo(new Complex(
			-juliaGenerator.getRealDimension() * .0625, 0));
		setJuliaComplexOrigin(newOrigin);
	    }
	});
	im.put(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 0),
		"Action_for_Left_Arrow_Key");

	// move right
	am.put("Action_for_Right_Arrow_Key", new AbstractAction() {
	    /** the generated serial version UID */
	    private static final long serialVersionUID = 6671663504426998800L;

	    @Override
	    public void actionPerformed(ActionEvent e) {
		Complex newOrigin
		        = new Complex(juliaGenerator.getComplexOrigin());
		newOrigin.addTo(new Complex(
			juliaGenerator.getRealDimension() * .0625, 0));
		setJuliaComplexOrigin(newOrigin);
	    }
	});
	im.put(KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 0),
		"Action_for_Right_Arrow_Key");

	// move up
	am.put("Action_for_Up_Arrow_Key", new AbstractAction() {
	    /** the generated serial version UID */
	    private static final long serialVersionUID = -6929086432023580556L;

	    @Override
	    public void actionPerformed(ActionEvent e) {
		Complex newOrigin
		        = new Complex(juliaGenerator.getComplexOrigin());
		newOrigin.addTo(new Complex(
			0, juliaGenerator.getRealDimension() * .0625));
		setJuliaComplexOrigin(newOrigin);
	    }
	});
	im.put(KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0),
		"Action_for_Up_Arrow_Key");

	// move down
	am.put("Action_for_Down_Arrow_Key", new AbstractAction() {
	    /** the generated serial version UID */
	    private static final long serialVersionUID = 2245326940677181354L;

	    @Override
	    public void actionPerformed(ActionEvent e) {
		Complex newOrigin
		        = new Complex(juliaGenerator.getComplexOrigin());
		newOrigin.addTo(new Complex(
			0, -juliaGenerator.getRealDimension() * .0625));
		setJuliaComplexOrigin(newOrigin);
	    }
	});
	im.put(KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 0),
		"Action_for_Down_Arrow_Key");

	// zoom in
	am.put("Action_for_+_Key", new AbstractAction() {
	    /** the generated serial version UID */
	    private static final long serialVersionUID = 3434995494597910699L;

	    @Override
	    public void actionPerformed(ActionEvent e) {
		setJuliaRealDimension(
			juliaGenerator.getRealDimension() * .875);
	    }
	});
	im.put(KeyStroke.getKeyStroke('+'), "Action_for_+_Key");

	// zoom out
	am.put("Action_for_-_Key", new AbstractAction() {
	    /** the generated serial version UID */
	    private static final long serialVersionUID = 8495499764904290734L;

	    @Override
	    public void actionPerformed(ActionEvent e) {
		setJuliaRealDimension(
			juliaGenerator.getRealDimension() * 1.142857);
	    }
	});
	im.put(KeyStroke.getKeyStroke('-'), "Action_for_-_Key");
	
	// show dialogue
	am.put("Action_for_Ctrl_Key", new AbstractAction() {
	    /** the generated serial version UID */
	    private static final long serialVersionUID = 8495499764904290734L;

	    @Override
	    public void actionPerformed(ActionEvent e) {
//		System.out.println("ctrl");
//		showDialogue(); // yet to be implemented
	    }
	});
	im.put(KeyStroke.getKeyStroke(KeyEvent.VK_CONTROL, 0),
	"Action_for_Ctrl_Key");
    }


    /**
     * Displays a dialogue in which the user can read and modify the parameters
     * of the currently displayed image.  Yet to be finished.
     */
//    private void showDialogue() {
//	JTextField[]  entries = new JTextField[8];
//	for (JTextField entry : entries) {
//	    entry = new JTextField(20);
//	}
//
//	entries[0].setText(String.valueOf(
//		this.mandelbrotGenerator.getComplexOrigin().re));
//
//	final JOptionPane optionPane = new JOptionPane(entries);
//	final JDialog dialog = new JDialog(this, "Options: ");
//	dialog.setContentPane(optionPane);
//	dialog.pack();
//	dialog.setVisible(true);
//    }


    /**
     * Paints the Mandelbrot fractal onto the given Graphics object.  The
     * dimensions of the given Graphics object must be equal to the dimensions
     * of the mandelbrotPanel.
     * 
     * @param g the given Graphics object
     */
    private void drawMandelbrotFractal(Graphics g) {
        for (int i = 0; i < mandelbrotPanelWidth; ++i) {
            for (int j = 0; j < mandelbrotPanelHeight; ++j) {
                int k = mandelbrotGenerator.getNumberOfIterationsAtPixel(i, j);
                g.setColor(mandelbrotColorizer.calculateColor(k));
                g.drawLine(i, j, i, j);
            }
        }

        if (this.dragPanelType == PanelType.MANDELBROT) {
            g.setColor(Color.GREEN);
            g.drawRect(Math.min(this.initialDragPosition.x,
        	            this.currentDragPosition.x),
        	    Math.min(this.initialDragPosition.y,
        		    this.currentDragPosition.y),
        	    Math.abs(this.initialDragPosition.x
        		    - this.currentDragPosition.x),
        	    Math.abs(this.initialDragPosition.y
        		    - this.currentDragPosition.y));
        }
    }

    /**
     * Paints the Julia fractal onto the given Graphics object.  The dimensions
     * of the given Graphics object must be equal to the dimensions of the
     * juliaPanel.
     * 
     * @param g the given Graphics object
     */
    private void drawJuliaFractal(Graphics g) {
        for (int i = 0; i < juliaPanelWidth; ++i) {
            for (int j = 0; j < juliaPanelHeight; ++j) {
                int k = juliaGenerator.getNumberOfIterationsAtPixel(i, j);
                g.setColor(juliaColorizer.calculateColor(k));
                g.drawLine(i, j, i, j);
            }
        }

        if (this.dragPanelType == PanelType.JULIA) {
            g.setColor(Color.GREEN);
            g.drawRect(Math.min(this.initialDragPosition.x,
        	            this.currentDragPosition.x),
        	    Math.min(this.initialDragPosition.y,
        		    this.currentDragPosition.y),
        	    Math.abs(this.initialDragPosition.x
        		    - this.currentDragPosition.x),
        	    Math.abs(this.initialDragPosition.y
        		    - this.currentDragPosition.y));
        }
    }

    private void updateMandelbrotTitle() {
        Complex mandelbrotOrigin = this.mandelbrotGenerator.getComplexOrigin();
        super.setTitle(String.format(
                "Mandelbrot Set  Origin: (%6f, %6f)  Diameter: %6f",
                mandelbrotOrigin.re, mandelbrotOrigin.im,
                this.mandelbrotGenerator.getRealDimension()));
    }

    private void updateJuliaTitle() {
        Complex juliaDefiningPoint = this.juliaGenerator.getDefiningPoint();
        Complex juliaOrigin = this.juliaGenerator.getComplexOrigin();
        this.juliaFrame.setTitle(String.format(
                "Julia Set for the Point (%6f, %6f)  Origin: (%6f, %6f)"
                        +  "  Diameter: %6f",
                juliaDefiningPoint.re, juliaDefiningPoint.im, juliaOrigin.re,
                juliaOrigin.im, this.juliaGenerator.getRealDimension()));
    }

    private void setMandelbrotComplexOrigin(Complex complexOrigin) {
        this.mandelbrotGenerator.setComplexOrigin(complexOrigin);
        this.updateMandelbrotTitle();
    }

    private void setMandelbrotRealDimension(double realDimension) {
        this.mandelbrotGenerator.setRealDimension(realDimension);
        this.updateMandelbrotTitle();
    }

    private void setJuliaDefiningPoint(Complex definingPoint) {
        this.juliaGenerator.setDefiningPoint(definingPoint);
        this.updateJuliaTitle();
    }

    private void setJuliaComplexOrigin(Complex complexOrigin) {
        this.juliaGenerator.setComplexOrigin(complexOrigin);
        this.updateJuliaTitle();
    }

    private void setJuliaRealDimension(double realDimension) {
        this.juliaGenerator.setRealDimension(realDimension);
        this.updateJuliaTitle();
    }

    // this method is necessary since the mandelbrotGenerator is not final
    private FractalGenerator getMandelbrotGenerator() {
        return mandelbrotGenerator;
    }

    // this method is necessary since the juliaGenerator is not final
    private JuliaFractalGenerator getJuliaGenerator() {
        return juliaGenerator;
    }
}
