/*
 * DoublePrecisionFractalGenerator
 * 
 * Version 0.2alpha
 * 
 * 2010-07-11
 * 
 * CopyLeft
 */
package infpp.fractal;

import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.LinkedList;

/**
 * The Threaded DoublePrecisionFractalGenerator class provides an abstract
 * class that serves as a common base for FractalGenerators using double
 * precision floating point arithmetic and multiple threads to speed up the
 * calculation.
 * 
 * @author both
 */
public abstract class ThreadedDoublePrecisionFractalGenerator
        extends DoublePrecisionFractalGenerator {
    private class FillThread extends Thread {
        private final FillThreadManagerThread parent;
        private final int[][]                 localWorkingBufferReference;
        private final int                     localWorkingBufferSize;
        private final Rectangle               fillArea;

        public FillThread(FillThreadManagerThread parent, int bufferIndex,
                int bufferSize, Rectangle fillArea) {
            this.parent = parent;
            this.localWorkingBufferReference = buffers.get(bufferIndex);
            this.localWorkingBufferSize = bufferSize;
            this.fillArea = fillArea;
        }

        @Override
        public void run() {
            super.setPriority(MIN_PRIORITY);
            for (int i = this.fillArea.x; this.parent.isFilling()
                    && i < this.fillArea.x + this.fillArea.width; ++i) {
                for (int j = this.fillArea.y; this.parent.isFilling()
                        && j < this.fillArea.y + this.fillArea.height; ++j) {
                    int[] screenCoordinates
                            = calculateScreenCoordinatesFromBufferCoordinates(
                                    localWorkingBufferSize, i, j);
                    Complex c = calculateComplexFromScreenCoordinates(
                            screenCoordinates[0], screenCoordinates[1]);
                    this.localWorkingBufferReference[i][j]
                            = calculateNumberOfIterationsAtComplex(c);
                }
            }
            this.parent.notifyOfFinishedThread(this);
        }
    }

    private class FillThreadManagerThread extends Thread {
        private ThreadedDoublePrecisionFractalGenerator parent;
        private boolean                filling = true;
        private LinkedList<FillThread> threads = new LinkedList<FillThread>();

        public FillThreadManagerThread(
                ThreadedDoublePrecisionFractalGenerator parent) {
            this.parent = parent;
        }

        public boolean isFilling() {
            return this.filling;
        }

        public void stopFilling() {
            this.filling = false;
        }

        public synchronized void notifyOfFinishedThread(FillThread thread) {
            this.threads.remove(thread);
            if (this.threads.isEmpty()) {
                this.notify();
            }
        }

        @Override
        public void run() {
            do {
                synchronized (parent) {
                    setCurrentWorkingBufferIndex(
                            currentWorkingBufferIndex + 1);

                    if (currentWorkingBufferIndex <= 1) {
                        this.threads.add(new FillThread(this,
                                currentWorkingBufferIndex,
                                currentWorkingBufferSize,
                                new Rectangle(0, 0, currentWorkingBufferSize,
                                        currentWorkingBufferSize)));
                    } else {
                        int requiredWidthFraction = Math.min(
                                currentWorkingBufferSize / 16,
                                getScreenWidth() / 16 + 1);
                        int requiredHeight = Math.min(currentWorkingBufferSize,
                                getScreenHeight());

                        if (currentWorkingBufferIndex == 2) {
                            for (int i = 0; i < 4; ++i) {
                                this.threads.add(new FillThread(this,
                                        currentWorkingBufferIndex,
                                        currentWorkingBufferSize,
                                        new Rectangle(
                                                i * 4 * requiredWidthFraction,
                                                0, 4 * requiredWidthFraction,
                                                requiredHeight)));
                            }
                        } else {
                            for (int i = 0; i < 16; ++i) {
                                this.threads.add(new FillThread(this,
                                        currentWorkingBufferIndex,
                                        currentWorkingBufferSize,
                                        new Rectangle(
                                                i * requiredWidthFraction,
                                                0, requiredWidthFraction,
                                                requiredHeight)));
                            }
                        }
                    }
                }

                synchronized (this) {
                    for (FillThread thread : this.threads) {
                        thread.start();
                    }

                    try {
                        this.wait();
                    } catch (InterruptedException e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
                }

                synchronized (parent) {
                    if (this.filling) {
                        setLastCompleteBufferIndex(currentWorkingBufferIndex);
                    }
                }
            } while (this.filling
                    && ((currentWorkingBufferSize <= getScreenWidth())
                            || (currentWorkingBufferSize
                                    <= getScreenHeight())));
        }
    }

    private final ArrayList<int[][]> buffers = new ArrayList<int[][]>();
    private int                      currentWorkingBufferIndex = 0;
    private int                      currentWorkingBufferSize  = 1;
    private int                      lastCompleteBufferIndex   = 0;
    private int                      lastCompleteBufferSize    = 1;
    private FillThreadManagerThread  currentFillThreadManagerThread;

    /**
     * Constructs a new ThreadedDoublePrecisionFractalGenerator and initializes
     * the buffers.
     */
    protected ThreadedDoublePrecisionFractalGenerator() {
        this.buffers.add(new int[1][1]);
    }

    /**
     * @see FractalGenerator#setComplexOrigin(Complex)
     */
    @Override
    public void setComplexOrigin(Complex complexOrigin) {
        super.setComplexOrigin(complexOrigin);
        this.startWorking();
    }

    /**
     * @see FractalGenerator#setRealDimension(double)
     */
    @Override
    public void setRealDimension(double realDimension) {
        super.setRealDimension(realDimension);
        this.startWorking();
    }

    /**
     * @see FractalGenerator#setScreenDimension(int, int)
     */
    @Override
    public void setScreenDimension(int width, int height) {
        super.setScreenDimension(width, height);
        this.startWorking();
    }

    /**
     * Returns the number of required iterations to determine divergence of a
     * certain sequence at the specified pixel converted to a complex number as
     * it is stored in one of the buffers used by this
     * ThreadedDoublePrecisionFractalGenerator.
     * 
     * @param  x the x coordinate of the specified pixel
     * @param  y the y coordinate of the specified pixel
     * @return the number of required iterations
     * @see FractalGenerator#getNumberOfIterationsAtPixel(int, int)
     */
    @Override
    public synchronized int getNumberOfIterationsAtPixel(int x, int y) {
	// synchronized might not be necessary
        int[] bufferCoordinates
                = calculateBufferCoordinatesFromScreenCoordinates(
                        this.lastCompleteBufferSize, x, y);
        return this.buffers.get(this.lastCompleteBufferIndex)
                [bufferCoordinates[0]][bufferCoordinates[1]];
    }

    /**
     * Performs the actual calculation of the number of required iterations to
     * determine divergence of a certain sequence defined using the specified
     * complex number.  In particular, this calculation is <I>not</I> performed
     * when calling getNumberOfIterationsAtPixel.
     * 
     * @param  c the specified complex number
     * @return the number of required iterations
     * @see    FractalGenerator#getNumberOfIterationsAtPixel(int, int)
     */
    protected abstract int calculateNumberOfIterationsAtComplex(Complex c);

    /** Begins the process of filling the buffers with data. */
    protected synchronized void startWorking() {
        if (this.currentFillThreadManagerThread != null) {
            this.currentFillThreadManagerThread.stopFilling();
        }
        setCurrentWorkingBufferIndex(0);

        this.currentFillThreadManagerThread
                = new FillThreadManagerThread(this);
        currentFillThreadManagerThread.start();
    }

    private synchronized void setLastCompleteBufferIndex(
            int lastCompleteBufferIndex) {
        if (this.buffers.size() <= lastCompleteBufferIndex) {
            int bufferSize = calculateBufferSizeFromBufferIndex(
                    lastCompleteBufferIndex);
            this.buffers.add(lastCompleteBufferIndex,
                    new int[bufferSize][bufferSize]);
        }

        this.lastCompleteBufferIndex = lastCompleteBufferIndex;
        this.lastCompleteBufferSize
                = calculateBufferSizeFromBufferIndex(lastCompleteBufferIndex);
    }

    private synchronized void setCurrentWorkingBufferIndex(
            int currentWorkingBufferIndex) {
        if (this.buffers.size() <= currentWorkingBufferIndex) {
            int bufferSize = calculateBufferSizeFromBufferIndex(
                    currentWorkingBufferIndex);
            this.buffers.add(currentWorkingBufferIndex,
                    new int[bufferSize][bufferSize]);
        }

        this.currentWorkingBufferIndex = currentWorkingBufferIndex;
        this.currentWorkingBufferSize = calculateBufferSizeFromBufferIndex(
                currentWorkingBufferIndex);
    }

    private int calculateBufferSizeFromBufferIndex(int bufferIndex) {
        return (int) Math.pow(4, bufferIndex);
    }

    private int[] calculateBufferCoordinatesFromScreenCoordinates(
            int bufferSize, int x, int y) {
        return new int[] {
                    (bufferSize >= super.getScreenWidth()) ? x
                            : x * bufferSize / super.getScreenWidth(),
                    (bufferSize >= super.getScreenHeight()) ? y
                            : y * bufferSize / super.getScreenHeight()
                };
    }

    private int[] calculateScreenCoordinatesFromBufferCoordinates(
            int bufferSize, int x, int y) {
        return new int[] {
                    (bufferSize >= super.getScreenWidth()) ? x
                            : x * super.getScreenWidth() / bufferSize,
                    (bufferSize >= super.getScreenHeight()) ? y
                            : y * super.getScreenHeight() / bufferSize
                };
    }
}
