/*
 * ThreadedDoublePrecisionJuliaGenerator
 * 
 * Version 0.2alpha
 * 
 * 2010-06-11
 * 
 * CopyLeft
 */
package infpp.fractal;

/**
 * The ThreadedDoublePrecisionJuliaGenerator provides an implementation for
 * calculating the number of required iterations to determine the divergence of
 * the sequence of points obtained by the recursion
 * z<sub>n + 1</sub> = (z<sub>n</sub>)<sup>2</sup> + c
 * where c is the point passed to the setDefiningPoint method and is constant
 * during the calculation of the fractal and z<sub>1</sub> is a point
 * determined by screen coordinates.  This implementation operates with
 * standard double precision arithmetic and uses multiple threads to speed up
 * the calculation.
 * 
 * @author both
 */
public class ThreadedDoublePrecisionJuliaGenerator
        extends ThreadedDoublePrecisionFractalGenerator
        implements JuliaFractalGenerator {
    private int maxIterations;
    private Complex definingPoint = new Complex(0, 0);

    /**
     * Constructs a new ThreadedDoublePrecisionJuliaGenerator.
     * 
     * @param maxIterations the maximum number of iterations to use
     */
    public ThreadedDoublePrecisionJuliaGenerator(int maxIterations) {
        super();
        this.maxIterations = maxIterations;
    }

    /**
     * @see JuliaFractalGenerator#getDefiningPoint()
     */
    @Override
    public Complex getDefiningPoint() {
	return this.definingPoint;
    }

    /**
     * @see JuliaFractalGenerator#setDefiningPoint(infpp.fractal.Complex)
     */
    @Override
    public void setDefiningPoint(Complex definingPoint) {
	this.definingPoint = definingPoint;
	this.startWorking();
    }

    /**
     * Calculates the number of required iterations to determine the divergence
     * of the sequence of points obtained by the recursion
     * z<sub>n + 1</sub> = (z<sub>n</sub>)<sup>2</sup> + c
     * where c is the point passed to the setDefiningPoint method and is
     * constant during the calculation of the fractal and z<sub>1</sub> is a
     * point determined by screen coordinates.
     *
     * @param  z the specified point on the complex plane
     * @return the number of required iterations
     * @see    ThreadedDoublePrecisionFractalGenerator
     *                 #calculateNumberOfIterationsAtComplex(
     *                         infpp.fractal.Complex)
     */
    @Override
    protected int calculateNumberOfIterationsAtComplex(Complex z) {
        int i;
        for (i = 0; i < this.maxIterations && z.getAbsSquare() <= 4; ++i) {
            z.square();
            z.addTo(this.getDefiningPoint());
        }
        return i;
    }
}
